/*
 * This file is part of C_Str object.
 * Copyright (C) 2005 - 2007 by HECHT Franck - franhec@gmail.com
 *
 * C_Str is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * C_Str is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with C_Str; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/* --------------------------------------------------------------------------

                      Gestion de l'objet string: C_Str
                                Version 2.1.0

   --------------------------------------------------------------------------
   Module   :  C_Str
   File     :  c_str.h

   Author   :  HECHT Franck (c) 2005 - 2007
   Creation :  18/11/2005
   Modified :  07/07/2007

   Description :  Objet de gestion de chaines de caracteres connaissant sa
                  propre taille.
   -------------------------------------------------------------------------- */

/* --------------------------------------------------------------------------

                                  Journal

   --------------------------------------------------------------------------
   Passage en version 2.0 :

   26/06/2007 :   Debut de la mise a jour complete de l'objet C_Str.

                  La mise a jour porte essentiellement sur de l'optimisation
                  des fonctions deja existantes et l'ajout de fonctions
                  supplementaires.

                  L'objet C_Str est a partir de cette version,
                  un type opaque !

   26/06/2007 :   Ajout des fonctions :
                     - C_Str_get
                     - C_Str_len

   27/06/2007 :   Ajout des fonctions :
                     - C_Str_nth_string_copy_obj
                     - C_Str_string_copy_obj
                     - C_Str_new_with_text

                  Par l'ajout de la fonction C_Str_new_with_text, la fonction
                  C_Str_new permet maintenant de crer des objets C_Str vides

                  Ajout de macros de substitution pour les fonctions
                  C_Str_is_word_in_string et C_Str_is_letter_in_string:
                     - C_Str_has_word
                     - C_Str_has_letter

   28/06/2007 :   Ajout des fonctions :
                     - C_Str_compare_ignore_case_obj
                     - C_Str_start_with_obj
                     - C_Str_end_with_obj

                  Ajout de macros de substitution pour les fonctions
                  C_Str_compare_ignore_case et C_Str_compare_ignore_case_obj:
                     - C_Str_compare_icase
                     - C_Str_compare_icase_obj

   29/06/2007 :   Ajout de la fonction :
                     - C_Str_new_from_file

   30/06/2007 :   Ajout des fonctions :
                     - C_Str_char_at
                     - C_Str_compare
                     - C_Str_compare_obj
                     - C_Str_compare_to_object
                     - C_Str_is_empty

                  Ajout d'une macro de substitution pour la fonction
                  C_Str_free_split_tab:
                     - C_Str_free_split

   Passage en version 2.1.0 :

   05/07/2007 :   Ajout de macros de subtitution :
                     - C_Str_ltrim
                     - C_Str_rtrim
                     - C_Str_trim

                  Ajout des fonctions :
                     - C_Str_printf
                     - C_Str_vprintf
                     - C_Str_fprintf
                     - C_Str_vfprintf
                     - C_Str_sprintf
                     - C_Str_vsprintf

   06/07/2007 :   Ajout des fonctions :
                     - C_Str_sub_string
                     - C_Str_sub_string_obj
                     - C_Str_sub_string_by_val
                     - C_Str_new_from_sub_string

                  Ajout des fonctions d'informations sur la bibliotheque :
                     - C_Str_get_id
                     - C_Str_get_version
                     - C_Str_get_version_major
                     - C_Str_get_version_minor
                     - C_Str_get_version_revision

   07/07/2007 :   Ajout des fonctions :
                     - C_Str_n_fill
   -------------------------------------------------------------------------- */


#ifndef H_HF_C_STR_181120051526
#define H_HF_C_STR_181120051526


#ifdef __cplusplus
extern "C"
{
#endif


#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>


/* macros =================================================================== */

#define C_Str_has_word(self, word)  \
   C_Str_is_word_in_string ((self), (word))


#define C_Str_has_letter(self, letter) \
   C_Str_is_letter_in_string ((self), (letter))


#define C_Str_compare_icase(self, string) \
   C_Str_compare_ignore_case ((self), (string))


#define C_Str_compare_icase_obj(self, compare) \
   C_Str_compare_ignore_case_obj ((self), (compare))


#define C_Str_free_split(tab) \
   C_Str_free_split_tab ((tab))


#define C_Str_ltrim(self)  \
   C_Str_lstrip ((self))


#define C_Str_rtrim(self)  \
   C_Str_rstrip ((self))


#define C_Str_trim(self)  \
   C_Str_strip ((self))


/* constants ================================================================ */

/* C_Str constants error. */
typedef enum
{
   C_STR_END_STR = -1,
   C_STR_NO_ERR = 0,
   C_STR_DYN_ALLOC_ERR,
   C_STR_NULL_ARG,
   C_STR_BAD_STR_LEN,
   C_STR_NB
}
C_Str_error;


/* types ==================================================================== */
/* structures =============================================================== */

/* C_Str opaque string object type. */
typedef struct _C_Str C_Str;


/* internal public functions ================================================ */

/* Library informations. */
const char * C_Str_get_id (void);
const char * C_Str_get_version (void);
int C_Str_get_version_major (void);
int C_Str_get_version_minor (void);
int C_Str_get_version_revision (void);


/* Constructors. */
C_Str * C_Str_new (void);
C_Str * C_Str_new_with_text (char const * const s_str);
C_Str * C_Str_new_from_file (char const * const s_file);
C_Str * C_Str_new_from_sub_string (char const * const s_str,
                                   size_t start,
                                   size_t end);


/* Destructors. */
void C_Str_destroy (C_Str ** const self);


/* Output functions. */
int C_Str_printf (C_Str * const format, ...);
int C_Str_vprintf (C_Str * const format, va_list args);
int C_Str_fprintf (FILE * file, C_Str * const format, ...);
int C_Str_vfprintf (FILE * file, C_Str * const format, va_list args);
int C_Str_sprintf (char * string, C_Str * const format, ...);
int C_Str_vsprintf (char * string, C_Str * const format, va_list args);


/* Publics functions. */
const char * C_Str_get (C_Str * const self);

size_t C_Str_len (C_Str * const self);

int C_Str_is_empty (C_Str * const self);

C_Str_error C_Str_cat_with_str (C_Str * const self,
                                char const * const s_str,
                                ...);

C_Str_error C_Str_cat (C_Str * const but, C_Str * const source, ...);

C_Str_error C_Str_nth_to_upper (C_Str * const self, size_t char_pos);

C_Str_error C_Str_nth_to_lower (C_Str * const self, size_t char_pos);

C_Str_error C_Str_to_upper (C_Str * const self);

C_Str_error C_Str_to_lower (C_Str * const self);

C_Str_error C_Str_capitalize (C_Str * const self);

C_Str_error C_Str_swap_case (C_Str * const self);

C_Str_error C_Str_lstrip (C_Str * const self);

C_Str_error C_Str_rstrip (C_Str * const self);

C_Str_error C_Str_strip (C_Str * const self);

C_Str_error C_Str_replace_string (C_Str * const self,
                                  size_t start,
                                  size_t lenght,
                                  char const * const s_str);

C_Str_error C_Str_replace_letter (C_Str * const self,
                                  const char old_letter,
                                  const char new_letter);

int C_Str_index (C_Str * const self, const char letter);

char C_Str_char_at (C_Str * const self, size_t index);

char * C_Str_nth_string_copy (C_Str * const self, int len);

C_Str * C_Str_nth_string_copy_obj (C_Str * const self, int len);

char * C_Str_string_copy (C_Str * const self);

C_Str * C_Str_string_copy_obj (C_Str * const self);

C_Str_error C_Str_cut (C_Str * const self, size_t start, size_t end);

size_t C_Str_how_much_letter (C_Str * const self, const char letter);

size_t C_Str_how_much_word (C_Str * const self, char const * const word);

int C_Str_is_word_in_string (C_Str * const self, char const * const word);

int C_Str_is_letter_in_string (C_Str * const self, const char letter);

char * C_Str_sub_string (C_Str * const self, const char * sub_string);

char * C_Str_sub_string_obj (C_Str * const self, C_Str * const sub_string);

char * C_Str_sub_string_by_val (C_Str * const self, size_t start, size_t end);

char ** C_Str_split (C_Str * const self, char const * const sep);

C_Str_error C_Str_free_split_tab (char *** tab);

int C_Str_compare_ignore_case (C_Str * const self, char const * const s_str);

int C_Str_compare_ignore_case_obj (C_Str * const self, C_Str * const compare);

int C_Str_compare (C_Str * const self, char const * const s_str);

int C_Str_compare_obj (C_Str * const self, C_Str * const compare);

int C_Str_compare_to_object (C_Str * const s1, C_Str * const s2);

C_Str_error C_Str_clear (C_Str * const self);

C_Str_error C_Str_fill (C_Str * const self, char const * const s_str);

C_Str_error C_Str_n_fill (C_Str * const self, char c, size_t len);

int C_Str_start_with (C_Str * const self, char const * const s_str);

int C_Str_start_with_obj (C_Str * const self, C_Str * const start);

int C_Str_end_with (C_Str * const self, char const * const s_str);

int C_Str_end_with_obj (C_Str * const self, C_Str * const end);

C_Str_error C_Str_reverse (C_Str * const self);

C_Str * C_Str_clone (C_Str * const self);


/* public variables ========================================================= */


#ifdef __cplusplus
}
#endif

#endif /* H_HF_C_STR_181120051526 */
